## From Julia you may load it using include("chinois.jl")
n = 1000 # taille du restaurant
m = 5000 # nombre de réalisations pour chaque valeur de theta
thetas = [1.] # valeurs de theta à simuler

function randewens(n,theta)
    # simule une réalisation de loi Ewens(n,theta) avec algo de Fisher-Yates-Knuth
    # (restaurants chinois, c'est-à-dire produit de transpositions spéciales)
    perm = [1:n]
    for i=2:n
        u = rand()
        p = theta/(theta+i-1)
        if (u > p)
                k = ceil((i-1)*(u-p)/(1-p))
                perm[i],perm[k] = perm[k],perm[i]
        end # if
    end # for 
    return perm
end # function

function findcycles(perm)
        # entrée : perm = permutation sous forme de vecteur ([1:l] désordonné)
        # sortie : array contenant des vector (un vector par cycle)
        l = length(perm)
        cycles = {}
        cycle = [1]
        seen = zeros(l)
        i = 1
        while (i <= l)
               if seen[i] == 0
                       seen[i] = 1
                       j = perm[i]
                       if j == cycle[1] # le cycle est bouclé
                               cycles = [cycles,{cycle}] # sauvegarde du cycle
                               i = cycle[1] +1
                               cycle = [i]
                       else
                               i = j
                               cycle = [cycle,i]
                       end # if
               else
                       i += 1
                       cycle = [i]
               end # if
         end # while
         return cycles
 end # function

# Main program.

Pkg.add("Gadfly") # produces "INFO: Nothing to be done." if already installed.
Pkg.add("Cairo") # produces "INFO: Nothing to be done." if already installed.
#Pkg.update()
using Gadfly
using Cairo
tables = zeros(length(thetas),m)
for t = 1:length(thetas)
    for i = 1:m
        perm = randewens(n,thetas[t])
        permcycles = findcycles(perm)
        tables[t,i] = length(permcycles)
    end
    @printf("theta=%2.2f, n=%i, theta*log(n)=%2.2f, m=%2.2f, v=%2.2f\n",
            thetas[t],n,thetas[t]*log(n),mean(tables[t,:]),var(tables[t,:]))
    p = plot(x=tables[t,:],Geom.histogram(bincount=25),
             Guide.xlabel("Longueur de cycle"),
             Guide.ylabel("Effectif"),
             Theme(default_color=color("grey"),panel_fill=color("white")))
    draw(PS(@sprintf("chinois-n-%i-theta-%2.2f.eps",n,thetas[t]), 10cm, 10cm), p)
end

#EOF
